// @ts-nocheck
import React, { useRef, useState } from 'react'
import { Descriptions, Modal, Tabs } from 'antd';
import classNames from 'classnames'
import type { ModalProps, DescriptionsProps, TabsProps } from 'antd'
import { get } from '@zhst/func';
import './index.less'
import Navigation from './components/navigation';

const DescriptionsItem = Descriptions.Item

export const componentPrefix = 'zhst-image'

export interface BigImageModalProps extends ModalProps {
  visible: boolean   // 弹框显示隐藏
  descriptionConfig: {
    data: {      // 描述列表
      title: string;        // 标题
      children: Pick<DescriptionsProps, 'items'>  // 描述项
    }[]
  }
  tabsConfig: {       // 导航栏配置
    data: Pick<TabsProps, 'items'> & {
      key: 'NORMAL' | 'COMPATER' | 'TRACK'
    }     // 导航栏列表
  }
  dataSource: any
  imageData: any
  relatedData: any
}

const initialStyle ={
  fontSize: '12px'
}

const BigImageModal: React.FC<BigImageModalProps> = (props) => {
  const {
    title = '-',
    open,
    visible,
    children,
    descriptionConfig = {
      data: []
    },
    tabsConfig = {
      data: [
        {
          label: '对比图模式',
          key: '1',
          children: '对比图组件',
        },
        {
          label: '场景图模式',
          key: '2',
          children: '场景图组件',
        }
      ],
    },
    dataSource = [],
    imageData = [],
    relatedData = [],
  } = props

  const showCropRef = useRef(false);
  const scaleRef = useRef(0);

  // ========================== 头切换 =========================
  const [activeKey, setActiveKey] = useState<string>(get(tabsConfig, 'data[0].key'));

  // ========================= 预览切换下标 =========================
  const [previewIndex, setPreviewIndex] = useState<number>(0)
  const [isRelated, setIsRelated] = useState<number>(false)

  return (
    <Modal
      destroyOnClose
      open={open || visible}
      footer={null}
      className={componentPrefix}
      title={title}
      {...props}
    >
      {descriptionConfig.data.map(descriptions => (
        <Descriptions
          key={descriptions.title}
          title={
            <p style={{ margin: '12px 0 0', fontSize: initialStyle.fontSize }}>
              {descriptions.title}
            </p>
          }
          column={8}
          style={{ padding: '0 64px' }}
        >
          {descriptions?.children?.map(item => (
            <DescriptionsItem
              key={item.key}
              label={item.label}
              span={1}
              contentStyle={{ fontSize: initialStyle.fontSize }}
              labelStyle={{ fontSize: initialStyle.fontSize }}
            >{item.children}</DescriptionsItem>
          ))}
        </Descriptions>
      ))}
      <Tabs
        defaultActiveKey={activeKey}
        centered
        tabBarStyle={{ fontSize: '18px' }}
        items={tabsConfig.data}
        {...tabsConfig}
      />
      <div
        className={classNames(`${componentPrefix}-view-container`)}
        style={
          activeKey === 'TRACK'
            ? {
                height: '718px',
                marginBottom: '0px',
              }
            : {}
        }
      >
        {
          activeKey !== 'TRACK' && (
            <>
              {/* ----------------------------------- 上一张按钮 ---------------------------------- */}
              <Navigation
                className={classNames(
                  `${componentPrefix}-view-container__nav`,
                  previewIndex <= 0 && `${componentPrefix}-view-container__nav--disabled`,
                  `${componentPrefix}-view-container__nav--left`
                )}
                show={isRelated ? (imageData.length > 1) : (dataSource.length > 1)}
                disabled={previewIndex <= 0}
                prev
                onClick={() => {
                  setPreviewIndex((pre) => pre - 1);
                }}
              />
              {/* ----------------------------------- 下一张按钮 ---------------------------------- */}
              <Navigation
                className={classNames(
                  `${componentPrefix}-view-container__nav`,
                  ((previewIndex >= imageData.length - 1) || (previewIndex >= dataSource.length - 1)) && `${componentPrefix}-view-container__nav--disabled`,
                  `${componentPrefix}-view-container__nav--right`
                )}
                show={isRelated ? (imageData.length > 1) : (dataSource.length > 1)}
                disabled={((previewIndex >= imageData.length - 1) || (previewIndex >= dataSource.length - 1))}
                next
                onClick={async (e) => {
                  setPreviewIndex((pre) => pre + 1);
                }}
              />
            </>
          )
        }
      </div>
    </Modal>
  )
}

export default BigImageModal
