/**
 * 适配老的大屏组件数据格式传入
 */
import React from 'react';
import { AlgorithmVersionStr, HumanProperty, ObjectType, Rect, ViewOption, AlignType, IScreenshotButtonProp, ODRECT } from './interface'

export type TAB_TYPE = 'COMPATER' | 'NORMAL' | 'TRACK';
export type MODEL_TYPE = 'VIDEO' | 'IMAGE';

export interface ImgViewProps extends React.HTMLAttributes<HTMLElement> {
  imageKey: string; //不在监听url变化 更新走销毁
  odRect: ODRECT;
  attachImg?: Array<{ label: string; url: string }>;
  showAttachImgLabel: boolean;
  /* 截图渲染 */
  screenshotButtonAlign: AlignType;
  screenshotButtonRender: (screenshotButtonProp: IScreenshotButtonProp) => React.ReactElement;
  /* 可观测值 */
  scale$?: number;
  showCrop$?: boolean;
  hideLeftTopBtn?: boolean;
  score?: number;
  viewOption?: ViewOption;
}

export interface VideoViewProps {
  /* 播放地址 */
  flvUrl: string;
  /* 播放总时间 */
  maxDuration?: number;
  /* 截图渲染 */
  screenshotButtonAlign?: AlignType;
  screenshotButtonRender?: (screenshotButtonProp: IScreenshotButtonProp) => React.ReactElement;
  /* 默认截图框 */
  defautlNormalizationRect?: Rect;
  /* 截图回调 */
  onCropChange?: (showCrop: boolean, normalizationRect: null | Rect) => void;

  /* 可观测值 */
  showCrop$?: boolean;
}

export interface CarouselProps {
  hasPre?: boolean;
  hasNext?: boolean;
  selectIndex: number;
  setSelectIndex: React.Dispatch<React.SetStateAction<number>>;
  dataSource: Array<{
    key: string;
    url: string;
  }>;
}

export type ISelectItem = Partial<Omit<ImgViewProps, 'screenshotButtonRender'>> &
  Partial<Omit<VideoViewProps, 'screenshotButtonRender'>>;

export interface HeaderProps {
  value: TAB_TYPE;
  onChange: (type: TAB_TYPE) => void;
  tabsFilter: TAB_TYPE[];
}

export interface ImgViewRef {
  /* 图片实例 */
  imgInsRef: React.MutableRefObject<any>;
  /* 切换图片模式 */
  setShowCrop: React.Dispatch<React.SetStateAction<boolean>>;
}


export interface VideoViewRef {
  /* 当前图片模式 */
  cropAble: boolean;
  setShowCrop: React.Dispatch<React.SetStateAction<boolean>>;
  downloadVideoframe: () => void;
}

export interface ParamProps {
  tab: string;
  selectItem: ISelectItem;
  imgViewRef: React.MutableRefObject<ImgViewRef>;
  VideoViewRef: React.MutableRefObject<VideoViewRef>;
  model: MODEL_TYPE;
  setModel: React.Dispatch<React.SetStateAction<MODEL_TYPE>>;
  /* 可观测值 */
  scale$: number;
  showCrop$: boolean;
}


export interface BigImageData {
  //imageKey 小图
  extendRectList: (Rect & { algorithmVersion: AlgorithmVersionStr; imageKey: string })[];
  rectList: (Rect & { algorithmVersion: AlgorithmVersionStr; imageKey: string })[];
  attachImg: { url: string; label: '形体' | '人脸' }[];
  odRect: Rect;
  imageKey: string; //大图
  imgSummary: string; //摘要图

  objectExtImageKey: string; //比对到的目标图扩展图 === imgSummary
  attributeList: { label: string; list: any[] }[];
  archiveImages?: any;
  spaceName: string;

  objectType: ObjectType;
  objectId: string; //这张摘要本身的Id
  bodyObjectId?: string;
  faceObjectId?: string; //这张摘要下的人脸Id(如果有)
  sourceObjectId?: string; //这张摘要上游的形体Id(如果有)
  cameraId: string;
  cameraName: string;
  selectIndex: number;
  humanProperty: HumanProperty;
  qualityScore?: number; //人脸质量分
  score: number; //相似度
  timestamp: string;
  bodyImageUrl: string;
  faceImageUrl: string;
  algorithmVersion: AlgorithmVersionStr;

  bodySpaceName: string;
  faceSpaceName: string;

  solutionId?: string;
  [index: string]: any;
}

interface IOldImageData {
  visible?: boolean;
  defaultModel?: MODEL_TYPE;
  onClose?: () => void;
  isLoading?: boolean;
  hasPre?: boolean;
  hasNext?: boolean;
  selectIndex?: number;
  onSelectIndexChange?: (i: number) => void;
  dataSource: any[];
  dataSources: any[];
  relatedData?: any[];
  transformPropFunc: (item: any) => ISelectItem;
  transformVideoPropFunc: (
    item: ISelectItem
  ) => Promise<Omit<VideoViewProps, 'screenshotButtonRender'>>;
  screenshotButtonRender?: (screenshotButtonProp: IScreenshotButtonProp) => React.ReactElement;
  showTool?: boolean;
  showCarousel?: boolean;
  imgViewProp?: Partial<ImgViewProps>;
  videoViewProp?: Partial<VideoViewProps>;
  ToolProps?: Partial<ToolProps>;
  nullDialogProp?: {
    emptyText?: string;
  };
  showHeader?: boolean;
  tabsFilter?: TAB_TYPE[];
  useVirtual?: boolean;
  loadNext?: (i: number) => Promise<void>;
  loadPre?: (i: number) => Promise<void>;
  children: React.ReactNode;
  title?: string;
  specialTitle?: string;
  isRelated?: boolean;
  carouselProp?: Partial<CarouselProps>;
}

export interface ToolProps {
  renderRight?: (props: ParamProps) => React.ReactNode;
  renderLeft?: (props: ParamProps) => React.ReactNode;
  renderVideoBtn?: boolean;
  param: ParamProps;
  disableVideo: boolean;
}

export default (data: IOldImageData) => {
  const newData = data

  return newData
}
