import doRequest from '../../utils/request';
import { NetworkType, useStores } from '@store/index';
import { useMount, useUnmount } from '@zhst/hooks';
import { useRef, useState } from 'react';
import type { CamerasStatusList, StatusList } from '../../interface'

const Streams = [
  'TASK_OVERVIEW_WAIT_START_STREAM',
  'TASK_OVERVIEW_WAIT_STREAM',
  'TASK_OVERVIEW_WAIT_START_SVINFER',
];

const StreamsNum = [1, 2, 3];
const RunsNum = [5, 9];
const ErrorsNum = [7];

const Runs = ['TASK_OVERVIEW_RUNNING', 'TASK_OVERVIEW_FINISH'];

const Errors = ['TASK_OVERVIEW_ERROR'];

export const getListTaskStatus = async (data?: { taskIndex: { deviceId: string }[] }) => {
  const res = await doRequest<{
    taskStatus: StatusList;
  }>({
    url: '/singer.TaskManagerService/ListTaskStatusResponse',
    method: 'POST',
    data: data || {}
  });
  return res;
};
export const getCameraStatusFunc = (list: StatusList, useNum = false) => {
  const _Runs: any = useNum ? RunsNum : Runs;
  const _Errors: any = useNum ? ErrorsNum : Errors;
  const _Streams: any = useNum ? StreamsNum : Streams;
  const res = list.reduce((res, cur) => {
    const cameraId = cur.taskIndex.deviceId;
    const status = cur.taskOverview;
    res.some((ids, i) => {
      if (ids.includes(cameraId)) {
        res[i] = ids.filter((id) => id !== cameraId);
        return true;
      }
      return false;
    });
    if (_Runs.includes(status)) {
      res[0].push(cameraId);
    } else if (_Errors.includes(status)) {
      res[1].push(cameraId);
    } else if (_Streams.includes(status)) {
      res[2].push(cameraId);
    }
    return res;
  }, [[], [], []]);
  return [...res];
};
let camerasStatusStoreList: CamerasStatusList = [[], [], []];
let num = 0;
const useTaskState = () => {
  const { store } = useStores();
  const [camerasStatusList, setCamerasStatusList] =
    useState<CamerasStatusList>(camerasStatusStoreList);
  const timerRef = useRef(null);

  const getCameraStatus = (list: StatusList, useNum = false) => {
    const res = getCameraStatusFunc(list, useNum);
    setCamerasStatusList(res);
  };
  const request = (data: { taskStatus: StatusList }[]) => {
    if (data) {
      const list: StatusList = [];
      data.forEach((v) => {
        list.push(...v.taskStatus);
      });
      getCameraStatus(list, store.networkType === NetworkType.NORMAL);
    }
  };
  useMount(async () => {
    num++;
    if (num - 1 > 0) {
      return;
    }
    const res = await getListTaskStatus();
    getCameraStatus(res.taskStatus);
    if (store.networkType === NetworkType.DUAL_NETWORK) {
      timerRef.current = setInterval(async () => {
        const res = await getListTaskStatus();
        request([res]);
      }, 2000);
    }
  });
  useUnmount(() => {
    clearInterval(timerRef.current);
    setCamerasStatusList([[], [], []]);
    num--;
    if (num === 0) {
      camerasStatusStoreList = [[], [], []];
    }
  });

  return camerasStatusList;
};

export default useTaskState;
