import { useMemo } from 'react';
import { throttle as loadshThrottle } from '@zhst/func';
import { useDeepEffect, useLatest } from '@zhst/hooks';
import ws from './ws';

const noop = (v: any) => null;
const getSelf = (v: any) => v;
export const SocketApi = {
  CameraTaskStatue: 'singer.DeviceService/SubScribeCameraTaskStatus',
  DeviceStatus: 'singer.TaskManagerService/SubscribeTaskStatus',
  SubscribeSolutionDeploy: 'singer.SolutionManagerService/SubscribeSolutionDeploy',
  SubscribeTasksSummary: 'singer.TaskManagerService/SubscribeTasksSummary',
  MonitorSubscribeResult: 'singer.MonitorService/MonitorSubscribeResult',
  MonitorSubscribeStatus: 'singer.MonitorService/MonitorSubscribeStatus',
  SubscribeArchiveGroupUpload: 'singer.ArchiveGroupService/CreateArchiveByImport',
  SubscribeJointTask: 'singer.JointTaskService/SubscribeJointTask',
  SubscribeGroupFragment: 'singer.VideoService/SubscribeVideoFragmentStatus', // 监听视频分片状态变化，包括新建、删除、变化
  SubscribeGroup: 'singer.VideoService/SubscribeGroup', // 监听视频分组状态变化，列表变化也通知
  SubscribeStreamEvent: 'singer.MediaManagerService/SubscribeStreamEvent', // 监听视频分组状态变化，列表变化也通知
};
type ApiKeys = keyof typeof SocketApi;

export default (
  topic: ApiKeys,
  iterator: any = noop,
  opt?: {
    req?: { [key: string]: any };
    throttle?: number;
    parseData?: boolean;
    beforeLoopTmp?: Function;
    shouldBreak: boolean;
    forceRefresh: any;
    close?: boolean;
  }
) => {
  const {
    req = {},
    throttle = 0,
    parseData = true,
    beforeLoopTmp = getSelf,
    shouldBreak = false,
    forceRefresh,
    close,
  } = opt || {};

  // 带上token
  const reqstring = useMemo(() => {
    const newReq = {
      ...req,
      extraHeaders: {
        authorization: localStorage.getItem('USER-TOKEN'),
      },
    };
    return JSON.stringify(newReq);
  }, [req]);

  const latestIterator = useLatest(iterator);
  useDeepEffect(() => {
    if (close) {
      return;
    }
    //控制socket 请求发送
    if (shouldBreak) {
      return;
    }
    //去抖动

    let tmpData: any = [];
    const throttleUpdate = loadshThrottle(() => {
      if (tmpData.length == 0) {
        return;
      }
      const _tmpData = beforeLoopTmp(tmpData);
      latestIterator.current(_tmpData); //加了throttle 数据就变成数组
      tmpData = [];
    }, throttle);

    const unSubscribe = ws.subscribe(SocketApi[topic], reqstring, (socketData: any) => {
      try {
        if (!throttle) {
          latestIterator.current(socketData);
        } else {
          tmpData.push(socketData);
          throttleUpdate();
        }
      } catch (error) {
        console.error('useSocke:', error);
      }
    });

    return () => {
      try {
        unSubscribe();
        throttleUpdate.cancel();
      } catch (error) {
        console.error(error);
        throw error;
      }
    };
  }, [topic, reqstring, shouldBreak, forceRefresh, close]);
};
