import { extend } from 'umi-request';
import type { RequestOptionsInit } from 'umi-request';
import { get } from '@zhst/func';
import { message } from 'antd';
import base64 from 'base-64';

export class ResponseError<D> extends Error {
  name;
  data;
  response;
  request;
  type;
  constructor(
    response: Response,
    text: string,
    data: D,
    request: {
      url: string;
      options: RequestOptionsInit;
    },
    type = 'ResponseError'
  ) {
    super(text || response.statusText);
    this.name = 'ResponseError';
    this.data = data;
    this.response = response;
    this.request = request;
    this.type = type;
  }
}

export const request = extend({
  getResponse: true,
  // timeout: 1000,
  parseResponse: false,
});

//错误处理中间件
request.use(async (ctx, next) => {
  const { req } = ctx;
  const { toast = true } = req?.options || {};
  try {
    await next();
    const { res } = ctx;
    const d = await res.text();
    if (res.status === 401) {
      message.warning('登录过期，请重新登录!');
      return;
    }
    const isEmptyRes = d === ''; //有些后端接口成功会返回空 做下兼容
    const body = !isEmptyRes ? JSON.parse(d) : d;
    if (res.status >= 200 && res.status < 300) {
      ctx.res = body;
    } else {
      // 先判断Grpc-Metadata-Errorx-Message
      let errMsg = res.headers.get('Grpc-Metadata-Errorx-Message');
      if (errMsg) {
        errMsg = window.utf8?.decode(base64.decode(errMsg));
        // 后判断 body中的message
      } else if (!errMsg && get(body, 'message')) {
        errMsg = `${get(body, 'message')}`;
      } else {
        // 最后看状态码
        errMsg = '您的网络发生异常，无法连接服务器';
      }
      toast && message.error(errMsg);
      throw new ResponseError(res, errMsg, d, req, 'CustomError');
    }
  } catch (error) {
    if (get(error, 'type') !== 'CustomError') {
      toast && message.error('您的网络发生异常，无法连接服务器');
    }
    throw error;
  }
});

export default request
