//@ts-nocheck
import { useState, useRef, useCallback, type CompositionEventHandler } from 'react';
import { isChrome } from '@zhst/func';
import type { IUseIMECompositionResult } from './interface'

const EMIT_CHANGE_AFTER_COMPOSITION_END = isChrome();

const defaultGetEventValue = (e) => e.target.value;

export default function useIMEComposition<OnChange extends (...args: any[]) => any>(
  propValue: string,
  onChangeProp?: OnChange,
  getEventValueProp?: (...args: any[]) => string,
  onCompositionStartProp?: CompositionEventHandler,
  onCompositionEndProp?: CompositionEventHandler
): IUseIMECompositionResult<OnChange> {
  const getEventValue = getEventValueProp || defaultGetEventValue;

  const isCompositionRef = useRef(false);
  const [compositionValue, setCompositionValue] = useState(propValue);

  const compositionValueRef = useRef(compositionValue);
  const onChangeRef = useRef(onChangeProp);
  const onCompositionStartRef = useRef(onCompositionStartProp);
  const onCompositionEndRef = useRef(onCompositionEndProp);
  compositionValueRef.current = compositionValue;
  onChangeRef.current = onChangeProp;
  onCompositionStartRef.current = onCompositionStartProp;
  onCompositionEndRef.current = onCompositionEndProp;

  // eslint-disable-next-line react-hooks/exhaustive-deps
  const onCompositionValueChange = useCallback(
    ((...args) => {
      if (isCompositionRef.current) {
        setCompositionValue(getEventValue(...args));
        // 若输入法正在输入，则不触发上层组件的事件
        return;
      }
      return onChangeRef.current?.(...args);
    }) as OnChange,
    [onChangeRef]
  );

  const onCompositionStart: CompositionEventHandler = useCallback(
    (e) => {
      isCompositionRef.current = true;
      onCompositionStartRef.current?.(e);
    },
    [onCompositionStartRef]
  );

  const onCompositionEnd: CompositionEventHandler = useCallback(
    (e) => {
      isCompositionRef.current = false;
      onCompositionEndRef.current?.(e);
      // chrome 的 onCompositionEnd 事件在 onChange 后触发，需要在 onCompositionEnd 后额外触发一次 onChange 事件
      if (EMIT_CHANGE_AFTER_COMPOSITION_END) {
        e.type = 'change';
        //todo: 此处优化
        onChangeRef.current?.(compositionValueRef.current, e);
      }
    },
    [onCompositionEndRef, onChangeRef]
  );

  // 只处理受控的组件
  const isControlled = propValue !== undefined;
  const passCompositionHandler = isControlled;
  const passCompositionValue = isControlled && isCompositionRef.current;

  return {
    value: passCompositionValue ? compositionValue : propValue,
    onChange: passCompositionHandler ? onCompositionValueChange : onChangeProp,
    onCompositionStart: passCompositionHandler ? onCompositionStart : onCompositionStartProp,
    onCompositionEnd: passCompositionHandler ? onCompositionEnd : onCompositionEndProp,
  };
}
