import { useLatest } from 'ahooks';
import events from 'events';
import { get, isArray } from '@zhst/func';
import { useEffect } from 'react';
import { useLocation, useNavigate } from 'react-router-dom';
import { JumpKey } from './constants'
import type { JumpType } from './interface'

const emitter = new events.EventEmitter();

const useJump = () => {
  const location = useLocation();
  const navigate = useNavigate();
  const jump = (path: string, data: { __key__: JumpType;[index: string]: any }) => {
    if (location.pathname !== path) {
      navigate(path, {
        state: { data, topic: JumpKey, __from__: location.pathname },
      });
    } else {
      emitter.emit(JumpKey, data);
    }
  };

  return jump;
};

export const useListenJump = (handle: (data: any) => void, type: JumpType | JumpType[]) => {
  const latestHandle = useLatest(handle);
  const navigate = useNavigate();
  const _type = isArray(type) ? type : [type];

  const location = useLocation();
  useEffect(() => {
    const { state, pathname, search } = location;
    if (state) {
      const { topic, data, __from__ } = state;
      if (topic === JumpKey && _type.includes(get(data, '__key__'))) {
        latestHandle.current({ ...data, __from__ });
      }
      navigate(`/${pathname}${search}`, {
        state: null,
      });
    }
  }, [location]);

  useEffect(() => {
    const handleEmit = (data: any) => {
      if (_type.includes(get(data, '__key__'))) {
        latestHandle.current(data);
      }
    };
    emitter.on(JumpKey, handleEmit);
    return () => {
      emitter.removeListener(JumpKey, handleEmit);
    };
  }, []);
};

export default useJump;
